library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.all;

entity main is
	port(
		BTN       : in  std_logic_vector(1 downto 0);
		NRES      : in  std_logic;
		LED       : out std_logic_vector(3 downto 0);
		SEG       : out std_logic_vector(7 downto 0);
		DISP      : out std_logic_vector(3 downto 0);
		CLK_10MHz : in  std_logic;
		TXD       : out std_logic;
		RXD       : in  std_logic
	);
end main;

architecture logic of main is
	component uart is
		port(
			from_uart_ready : in  std_logic                    := 'X'; -- ready
			from_uart_data  : out std_logic_vector(7 downto 0); -- data
			from_uart_error : out std_logic; -- error
			from_uart_valid : out std_logic; -- valid
			to_uart_data    : in  std_logic_vector(7 downto 0) := (others => 'X'); -- data
			to_uart_error   : in  std_logic                    := 'X'; -- error
			to_uart_valid   : in  std_logic                    := 'X'; -- valid
			to_uart_ready   : out std_logic; -- ready
			clk             : in  std_logic                    := 'X'; -- clk
			UART_RXD        : in  std_logic                    := 'X'; -- RXD
			UART_TXD        : out std_logic; -- TXD
			reset           : in  std_logic                    := 'X' -- reset
		);
	end component uart;
	type state_t is (IDLE, READ_DONE, WRITING);
	signal state           : state_t;
	signal read_data_buf   : std_logic_vector(7 downto 0);
	signal write_data_buf  : std_logic_vector(7 downto 0);
	signal from_uart_valid : std_logic;
	signal from_uart_ready : std_logic;
	signal to_uart_valid   : std_logic;
	signal to_uart_ready   : std_logic;
	signal rst             : std_logic;
	signal clk_1kHz        : std_logic;
	constant dots          : std_logic_vector(3 downto 0) := (others => '1');
	signal display_num     : std_logic_vector(15 downto 0);
begin
	rst <= not NRES;

	uart1 : uart port map(
			reset           => rst,
			clk             => CLK_10MHz,
			from_uart_ready => from_uart_ready,
			from_uart_data  => read_data_buf,
			from_uart_valid => from_uart_valid,
			to_uart_data    => write_data_buf,
			to_uart_valid   => to_uart_valid,
			to_uart_ready   => to_uart_ready,
			UART_TXD        => TXD,
			UART_RXD        => RXD);

	process(rst, CLK_10MHz)
	begin
		if rst = '1' then
			state           <= IDLE;
			to_uart_valid   <= '0';
			from_uart_ready <= '0';
		elsif rising_edge(CLK_10Mhz) then
			case state is
				when IDLE =>
					if from_uart_valid = '1' then
						write_data_buf  <= read_data_buf;
						from_uart_ready <= '1';
						state           <= READ_DONE;
					end if;
				when READ_DONE =>
					from_uart_ready <= '0';
					if to_uart_ready = '1' then
						to_uart_valid <= '1';
						state         <= WRITING;
					end if;
				when others =>
					to_uart_valid <= '0';
					state         <= IDLE;
			end case;
		end if;
	end process;

	dv1kHz : entity divider generic map(10000) port map(CLK_10MHz, rst, clk_1kHz);
	md : entity work.multi_display port map(display_num, dots, clk_1kHz, SEG, DISP);
	display_num(15 downto 8) <= (others => '0');
	display_num(7 downto 0)  <= write_data_buf;

end architecture;